@@ -1,12 +1,6 @@
 //! This is a simple crate that makes making qwac games easier.
-//!
-//! It has one feature, `alloc`.  You should enable it if you plan on using liballoc (including if
-//! `std` is enabled), otherwise, leave it disabled to save some space.
 
-#![no_std]
-
-#[cfg(feature = "alloc")]
-pub extern crate alloc;
+#![cfg_attr(not(feature = "std"), no_std)]
 
 mod ffi;
 
@@ -22,6 +16,10 @@ pub use canvas::Screen;
 pub use event::Event;
 pub use texture::Texture;
 
+/// Just represents the unit for euclidean transforms.
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub struct Pixels;
+
 #[cfg(feature = "f32-canvas")]
 pub type GridUnit = f32;
 
@@ -31,56 +29,22 @@ pub type GridUnit = f64;
 #[cfg(not(any(feature = "f32-canvas", feature = "f64-canvas")))]
 pub type GridUnit = i32;
 
-#[derive(Default, Clone, Copy, Debug, PartialEq, PartialOrd)]
-pub struct GridVector {
-    pub x: GridUnit,
-    pub y: GridUnit,
-}
-
-impl GridVector {
-    pub fn new(x: GridUnit, y: GridUnit) -> Self {
-        Self { x, y }
-    }
-}
-
-impl From<(GridUnit, GridUnit)> for GridVector {
-    fn from(units: (GridUnit, GridUnit)) -> Self {
-        Self {
-            x: units.0,
-            y: units.1,
-        }
-    }
-}
-
-impl From<GridVector> for (GridUnit, GridUnit) {
-    fn from(vector: GridVector) -> Self {
-        (vector.x, vector.y)
-    }
-}
-
-#[derive(Default, Clone, Copy, Debug, PartialEq, PartialOrd)]
-pub struct Rectangle {
-    pub origin: GridVector,
-    pub size: GridVector,
-}
-
-impl Rectangle {
-    pub fn new(origin: GridVector, size: GridVector) -> Self {
-        Self { origin, size }
-    }
-}
+pub type Vector = euclid::Vector2D<GridUnit, Pixels>;
+pub type Rect = euclid::Rect<GridUnit, Pixels>;
+pub type Size = euclid::Size2D<GridUnit, Pixels>;
+pub type Point = euclid::Point2D<GridUnit, Pixels>;
 
 #[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
-pub struct Color {
+pub struct Rgba {
     pub r: u8,
     pub g: u8,
     pub b: u8,
     pub a: u8,
 }
 
-impl Color {
-    pub fn white() -> Color {
-        Color {
+impl Rgba {
+    pub fn white() -> Rgba {
+        Rgba {
             r: u8::MAX,
             g: u8::MAX,
             b: u8::MAX,
@@ -88,8 +52,8 @@ impl Color {
         }
     }
 
-    pub fn black() -> Color {
-        Color {
+    pub fn black() -> Rgba {
+        Rgba {
             r: u8::MIN,
             g: u8::MIN,
             b: u8::MIN,
@@ -97,8 +61,8 @@ impl Color {
         }
     }
 
-    pub fn clear() -> Color {
-        Color {
+    pub fn clear() -> Rgba {
+        Rgba {
             r: u8::MIN,
             g: u8::MIN,
             b: u8::MIN,
@@ -122,14 +86,14 @@ pub trait Game {
 #[macro_export]
 macro_rules! qwac_game {
     ($game_type:ty) => {
-        #[cfg(not(feature = "alloc"))]
+        #[cfg(not(feature = "std"))]
         static mut QWAC_RUST_GAME: Option<$game_type> = None;
 
         #[no_mangle]
         pub extern "C" fn qwac_game_create(timestamp: f64) -> *mut $game_type {
             let game = <$game_type as $crate::Game>::new(timestamp);
 
-            #[cfg(not(feature = "alloc"))]
+            #[cfg(not(feature = "std"))]
             {
                 unsafe {
                     QWAC_RUST_GAME = None;
@@ -138,8 +102,8 @@ macro_rules! qwac_game {
                 core::ptr::null_mut()
             }
 
-            #[cfg(feature = "alloc")]
-            $crate::alloc::boxed::Box::new(game).into_raw()
+            #[cfg(feature = "std")]
+            Box::new(game).into_raw()
         }
 
         #[no_mangle]
@@ -149,10 +113,10 @@ macro_rules! qwac_game {
             button: $crate::event::GamepadButton,
             pressed: u8,
         ) {
-            #[cfg(not(feature = "alloc"))]
+            #[cfg(not(feature = "std"))]
             let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };
 
-            #[cfg(feature = "alloc")]
+            #[cfg(feature = "std")]
             let game = unsafe { &mut *_game };
 
             let state = if pressed == 0 {
@@ -173,10 +137,10 @@ macro_rules! qwac_game {
 
         #[no_mangle]
         pub extern "C" fn qwac_game_update(_game: *mut $game_type, frame_milliseconds: u32) {
-            #[cfg(not(feature = "alloc"))]
+            #[cfg(not(feature = "std"))]
             let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };
 
-            #[cfg(feature = "alloc")]
+            #[cfg(feature = "std")]
             let game = unsafe { &mut *_game };
 
             $crate::Game::update(game, frame_milliseconds);
@@ -184,14 +148,14 @@ macro_rules! qwac_game {
 
         #[no_mangle]
         pub extern "C" fn qwac_game_destroy(_game: *mut $game_type) {
-            #[cfg(not(feature = "alloc"))]
+            #[cfg(not(feature = "std"))]
             unsafe {
                 QWAC_RUST_GAME = None;
             }
 
-            #[cfg(feature = "alloc")]
+            #[cfg(feature = "std")]
             unsafe {
-                $crate::alloc::boxed::Box::from_raw(game);
+                Box::from_raw(game);
             }
         }
     };
